package com.company.crossdatastore;

import io.jmix.autoconfigure.data.JmixLiquibaseCreator;
import io.jmix.core.JmixModules;
import io.jmix.core.Resources;
import io.jmix.data.impl.JmixEntityManagerFactoryBean;
import io.jmix.data.impl.JmixTransactionManager;
import io.jmix.data.persistence.DbmsSpecifics;
import jakarta.persistence.EntityManagerFactory;
import liquibase.integration.spring.SpringLiquibase;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.autoconfigure.liquibase.LiquibaseProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.JpaVendorAdapter;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;

import javax.sql.DataSource;

@Configuration
public class Ds2StoreConfiguration {

    @Bean
    @ConfigurationProperties("ds2.datasource")
    DataSourceProperties ds2DataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean
    @ConfigurationProperties(prefix = "ds2.datasource.hikari")
    DataSource ds2DataSource(@Qualifier("ds2DataSourceProperties") DataSourceProperties properties) {
        return properties.initializeDataSourceBuilder().build();
    }

    @Bean
    LocalContainerEntityManagerFactoryBean ds2EntityManagerFactory(
            @Qualifier("ds2DataSource") DataSource dataSource,
            JpaVendorAdapter jpaVendorAdapter,
            DbmsSpecifics dbmsSpecifics,
            JmixModules jmixModules,
            Resources resources
    ) {
        return new JmixEntityManagerFactoryBean("ds2", dataSource, jpaVendorAdapter, dbmsSpecifics, jmixModules, resources);
    }

    @Bean
    JpaTransactionManager ds2TransactionManager(@Qualifier("ds2EntityManagerFactory") EntityManagerFactory entityManagerFactory) {
        return new JmixTransactionManager("ds2", entityManagerFactory);
    }

    @Bean("ds2LiquibaseProperties")
    @ConfigurationProperties(prefix = "ds2.liquibase")
    public LiquibaseProperties ds2LiquibaseProperties() {
        return new LiquibaseProperties();
    }

    @Bean("ds2Liquibase")
    public SpringLiquibase ds2Liquibase(@Qualifier("ds2DataSource") DataSource dataSource,
                                        @Qualifier("ds2LiquibaseProperties") LiquibaseProperties liquibaseProperties) {
        return JmixLiquibaseCreator.create(dataSource, liquibaseProperties);
    }
}
